# vim:tabstop=4:shiftwidth=4:softtabstop=0:noexpandtab

define PACKAGE_COPY_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/package/$(notdir $(1)) : $(1)
	@echo $$(notdir $$@)
	$(quiet)cp $$< $$@
endef

define COPY_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)): $(2)
	@echo $$<
	$(quiet)$(MKDIR) -p $$(dir $$@)
	$(quiet)cp -f $$< $$@
endef

define PREPROCESS_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)): \
  $(2) $(WORK_DIRECTORY)/$(PROJECT_NAME)/project/.makefile
	@echo $$<
	$(quiet)$(MKDIR) -p $$(dir $$@)
	$(veryquiet)$(CC) -E -P -x c -C -Wundef -Werror $(C_DEFINES) $$< | sed -e 's/^[ \t]*//g' | sed -e 's/&apos;/\x27/g' > $$@
endef

define GZIP_template
$(1)_EMBEDDED_FILES = $(subst $(2),$(2).gz,$($(1)_EMBEDDED_FILES))
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)).gz: $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2))
	$(quiet)gzip -c $$< > $$<.gz
endef

define BUILD_FILE_STRUCT_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/c/$(call PRUNE_TO_MODULE_DIRECTORY,$(2)).c: \
  $(WORK_DIRECTORY)/$(PROJECT_NAME)/modules/$(1)/embedded/files/$(call PRUNE_TO_MODULE_DIRECTORY,$(2))
	@echo $$<
	@$(MKDIR) -p $$(dir $$@)
	@echo "/* Generated by makefile */" > $$@
	@echo "#include \"ltrx_embedded_files.h\"" >> $$@
	@echo "const struct ltrx_embedded_file Embedded_file_$(subst /,_slash_,$(subst .,_dot_,$(call PRUNE_TO_MODULE_DIRECTORY,$(2))))_contents = {" >> $$@
	@echo "\"$(call PRUNE_TO_MODULE_DIRECTORY,$(2))\"", >> $$@
	@echo "`stat -c%s $$<`," >>$$@
	@od -An -bv $$< | sed -e 's/^/"/g' -e 's/$$$$/"/g' -e 's/ /\\/g' >> $$@
	@echo -e "\"\"\n};" >> $$@
endef

define C_DEPENDENCY_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/dependencies/$(call MANGLE,$(1:.c=.dep)) : \
  $(1) \
  $(ALL_GENERATED_HEADERS)
	@echo $$(notdir $$@)
	@$(RM) $$@ $$@.$$$$
	$(quiet)$(CC) -MM -MF '$$@.$$$$' -MQ '$$@' $(call DEPENDENCY_INCLUDES,$(1)) $(addprefix -D,$(strip $(FEATURE_FLAGS) $(EVALUATED_FLAGS) $(VARDEF_NUMBER_RANGE_FLAGS))) $$<
	$(veryquiet)sed -e "s,\([a-zA-Z]\)\\:\/,\/\1\/,g" -e "s,$*[_a-z/A-Z.0-9-]*\.dep[ :]*,$(WORK_DIRECTORY)/$(PROJECT_NAME)/objects/$(call MANGLE,$(1:.c=.o)) $$@ : ," < $$@.$$$$ > $$@
	@$(RM) $$@.$$$$
-include $(WORK_DIRECTORY)/$(PROJECT_NAME)/dependencies/$(call MANGLE,$(1:.c=.dep))
endef

define C_SOURCE_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/objects/$(call MANGLE,$(1:.c=.o)) : \
  $(1) \
  $(ALL_GENERATED_HEADERS)
	@echo $$(notdir $$@)
	$(quiet)$(CC) $$(CFLAGS) $$(WARNFLAGS) $(call DEPENDENCY_INCLUDES,$(1)) $(C_DEFINES) -c $$< -o $$@
endef

define C_SOURCE_check_template
$(1) :
endef

define ASM_SOURCE_template
$(WORK_DIRECTORY)/$(PROJECT_NAME)/objects/$(call MANGLE,$(1:.s=.o)) : \
  $(1)
	@echo $$(notdir $$@)
	$(quiet)$(AS) $$(ASMFLAGS) -c $$< -o $$@
endef
